Package: spray
Type: Package
Title: Sparse Arrays and Multivariate Polynomials
Version: 1.0-26
Authors@R: person(given=c("Robin", "K. S."), family="Hankin",
           role = c("aut","cre"),
	   email="hankin.robin@gmail.com",
	   comment = c(ORCID = "0000-0001-5982-0415"))
Maintainer: Robin K. S. Hankin <hankin.robin@gmail.com>
Description: Sparse arrays interpreted as multivariate polynomials.
    Uses 'disordR' discipline (Hankin, 2022,
    <doi:10.48550/ARXIV.2210.03856>).  To cite the package in
    publications please use Hankin (2022) <doi:10.48550/ARXIV.2210.10848>.
License: GPL (>= 2)
Depends: methods
Suggests: polynom, testthat, covr
Imports: Rcpp (>= 0.12.3), partitions, magic, disordR (>= 0.9-6),
        stringr
LinkingTo: Rcpp
URL: https://github.com/RobinHankin/spray,
        https://robinhankin.github.io/spray/
BugReports: https://github.com/RobinHankin/spray/issues
Author: Robin K. S. Hankin [aut, cre] (<https://orcid.org/0000-0001-5982-0415>)
Built: R 4.4.1; x86_64-apple-darwin21.6.0; 2024-08-20 16:19:42 UTC; unix
